package com.qr.flutter_toolkit.video_thumbnail;

import android.annotation.SuppressLint;
import android.graphics.Bitmap;
import android.media.MediaMetadataRetriever;
import android.util.Log;

import androidx.annotation.NonNull;

import java.io.ByteArrayOutputStream;
import java.io.FileOutputStream;
import java.util.HashMap;
import java.util.Map;

import io.flutter.plugin.common.MethodCall;
import io.flutter.plugin.common.MethodChannel;
import io.reactivex.Observable;
import io.reactivex.android.schedulers.AndroidSchedulers;
import io.reactivex.schedulers.Schedulers;

public class VideoThumbnailHandler implements MethodChannel.MethodCallHandler {
    private static String TAG = VideoThumbnailHandler.class.getSimpleName();

    @SuppressLint("CheckResult")
    @Override
    public void onMethodCall(@NonNull MethodCall methodCall, @NonNull MethodChannel.Result result) {
        final Map<String, Object> args = methodCall.arguments();
        final String video = (String) args.get("video");
        final int format = (int) args.get("format");
        final int maxhow = (int) args.get("maxhow");
        final int timeMs = (int) args.get("timeMs");
        final int quality = (int) args.get("quality");
        if (methodCall.method.equals("file")) {
            Observable.fromCallable(() -> {
                return buildThumbnailFile(video, format, maxhow, timeMs, quality);
            }).subscribeOn(Schedulers.io())
                    .observeOn(AndroidSchedulers.mainThread())
                    .subscribe(s -> {
                        result.success(s);
                    }, throwable -> {
                        result.error("VideoThumbnail Error", throwable.getMessage(), null);
                    });
        } else if (methodCall.method.equals("data")) {
            Observable.fromCallable(() -> {
                return buildThumbnailData(video, format, maxhow, timeMs, quality);
            }).subscribeOn(Schedulers.io())
                    .observeOn(AndroidSchedulers.mainThread())
                    .subscribe(bytes -> {
                        result.success(bytes);
                    }, throwable -> {
                        result.error("VideoThumbnail Error", throwable.getMessage(), null);
                    });
        } else {
            result.notImplemented();
        }
    }


    private static Bitmap.CompressFormat intToFormat(int format) {
        switch (format) {
            default:
            case 0:
                return Bitmap.CompressFormat.JPEG;
            case 1:
                return Bitmap.CompressFormat.PNG;
            case 2:
                return Bitmap.CompressFormat.WEBP;
        }
    }

    private static String formatExt(int format) {
        switch (format) {
            default:
            case 0:
                return "jpg";
            case 1:
                return "png";
            case 2:
                return "webp";
        }
    }

    private byte[] buildThumbnailData(String vidPath, int format, int maxhow, int timeMs, int quality) {
        Log.d(TAG, String.format("buildThumbnailData( format:%d, maxhow:%d, timeMs:%d, quality:%d )", format, maxhow,
                timeMs, quality));
        Bitmap bitmap = createVideoThumbnail(vidPath, maxhow, timeMs);
        if (bitmap == null)
            throw new NullPointerException();

        ByteArrayOutputStream stream = new ByteArrayOutputStream();
        bitmap.compress(intToFormat(format), quality, stream);
        bitmap.recycle();
        return stream.toByteArray();
    }

    private String buildThumbnailFile(String vidPath, int format, int maxhow, int timeMs, int quality) {
        Log.d(TAG, String.format("buildThumbnailFile( format:%d, maxhow:%d, timeMs:%d, quality:%d )", format, maxhow,
                timeMs, quality));
        final byte[] bytes = buildThumbnailData(vidPath, format, maxhow, timeMs, quality);
        final String ext = formatExt(format);
        final int i = vidPath.lastIndexOf(".");
        String fullpath = vidPath.substring(0, i + 1) + ext;

        try {
            FileOutputStream f = new FileOutputStream(fullpath);
            f.write(bytes);
            f.close();
            Log.d(TAG, String.format("buildThumbnailFile( written:%d )", bytes.length));
        } catch (java.io.IOException e) {
            e.getStackTrace();
            throw new RuntimeException(e);
        }
        return fullpath;
    }

    private static Bitmap createVideoThumbnail(String video, int targetSize, int timeMs) {
        Bitmap bitmap = null;
        MediaMetadataRetriever retriever = new MediaMetadataRetriever();
        try {
            Log.d(TAG, String.format("setDataSource: %s )", video));
            if (video.startsWith("file://") || video.startsWith("/")) {
                retriever.setDataSource(video);
            } else {
                retriever.setDataSource(video, new HashMap<String, String>());
            }

            if (targetSize != 0) {
                if (android.os.Build.VERSION.SDK_INT >= 27) {
                    // API Level 27
                    bitmap = retriever.getScaledFrameAtTime(timeMs * 1000, 0, targetSize, targetSize);
                } else {
                    bitmap = retriever.getFrameAtTime(timeMs * 1000);
                    if (bitmap != null) {
                        int width = bitmap.getWidth();
                        int height = bitmap.getHeight();
                        int max = Math.max(width, height);
                        float scale = (float) targetSize / max;
                        int w = Math.round(scale * width);
                        int h = Math.round(scale * height);
                        Log.d(TAG, String.format("original w:%d, h:%d, scale:%6.4f => %d, %d", width, height, scale, w,
                                h));
                        bitmap = Bitmap.createScaledBitmap(bitmap, w, h, true);
                    }
                }
            } else {
                bitmap = retriever.getFrameAtTime(timeMs * 1000);
            }
        } catch (IllegalArgumentException ex) {
            ex.printStackTrace();
        } catch (RuntimeException ex) {
            ex.printStackTrace();
        } finally {
            try {
                retriever.release();
            } catch (RuntimeException ex) {
                ex.printStackTrace();
            }
        }

        return bitmap;
    }
}
